#!/usr/bin/env Rscript
#------------------- READ RAMD DATA
getdata <- function(name) {
        if(file.info(name)$size == 0) {
              stop("Wrong format of the RAMD output data file (file is empty)")
        }
	s <- read.table(name,header = FALSE)
	x1 <- s[["V2"]]
        if (is.numeric(x1) == FALSE) {
              stop("Wrong format of the RAMD output data file (the second column must contain integer numbers)")
           }
     return (x1/500000.0)  # transfer MD steps to ns
}


#---------------------- GET RAMD TIME
gettime <- function(xtot) {
      htot_norm <- sort(xtot, decreasing = TRUE)
      k<- 1
      for (i in seq_along(htot_norm)) {
        if (k > length (htot_norm)/2.0) break  # residence time - when compound dissociated in 50% of trajectories 
        k <- k+1
      }
      restime <- (htot_norm[k-1]+htot_norm[k+1])/2.0
      return (restime)
}

#-------------------- BOOTSTRAPPING DATA using 80% of the dataset ==========
getbootstr  <- function(x) {
    data <- gettime(x)
    for (i in 1:200) {
            y <-sample(x, length(x)*0.8,replace=T)
    		data <- c(data,gettime(y))
    }
    return (data)
}

#=======================================
#--------------- compute residence time  from SEVERAL RAMD OUTPUT FILES (replicas)
#----------------- COMPUTE STATISTICS

one_compound <- function (input,compN="") {
  xmax=8
  # pre-process data 
fun_preprocess <-function(filelist) {
    datainp <- NULL
    xmax <- NULL
    s <- 0
    for (k in seq_along(filelist)) {
      datainp <- c(datainp,getdata(filelist[k]))
      s <- s+1
    }
    if(length(datainp) > 0) {
      xmax=max(datainp,na.rm = TRUE)
    }
 #   print (xmax)
    return(xmax)
  }

#=====================================================
# compute statistics for each replica separately and retern list of residence times for each replica  and their SD
fun_prot_part <- function(filelist,compN="") {
 list_out <-c()
 i <- 1
 for (k in seq_along(filelist)) {
    data_inp <- getdata(filelist[k])
    x1=getbootstr(data_inp)
    result <-shapiro.test(data_inp)
    out=paste(MEAN=c(mean(x1),SD=sd(x1)))
    list_out <- c(list_out,mean(x1),sd(x1))
    cmean_n <- toString(round(c(mean(x1)),digits = 2))
    sd_n <- toString(round(sd(x1),digits = 2))
    out <- paste("t=",cmean_n,"+-",sd_n,"(",round(100*sd(x1)/mean(x1),0),"%)", sep="")
    print (paste(toString(k),":",filelist[k]," (",toString(length(data_inp)),"trj) ",out, sep=""))
    #normal distribution
    i <- i+1
 }
  return (list_out)
}

# compute statistics for all replicas together
fun_prot_tot <- function(filelist,compN) {
 xall <- NULL
 for (k in seq_along(filelist)) {
    xall <- c(xall,getdata(filelist[k]))
 }  
    x1 <- getbootstr(xall)
    return(c(mean(x1),sd(x1)))

}
 file_list <- input
 if (length(file_list) < 1) {
   print ("files are missing")
 }
 xmax<-fun_preprocess(file_list)
 number_files=length(file_list)+2

 l <- fun_prot_part(file_list,compN)
 d <- fun_prot_tot(file_list,"")
 l_length <- ((length(l))/2)
 ml <-as.data.frame(matrix(l,ncol=2,byrow=TRUE,dimnames = list(time=c(1:l_length), name=c("mean", "sd"))))
 max_time <- max(ml[["mean"]])
 file_list_top <- NULL
 
 a1 <- mean(ml[["mean"]])  # average residence time- mean residence time over all replicas
 a2 <- sd(ml[["mean"]])   #  SD of the average residence time
 a3 <- mean(ml[["sd"]])  # mean SD of the residence times over all replicas
 print(paste("MEAN (over all replicas):",round(a1,2),sep=""))
 print(paste("SD (over all replicas):",round(a2,2),"(",round(100*a2/a1,0),"%); Mean over SD in replicas:",round(a3,2),sep=""))
 if(is.na(a2)){ rmsd <- a3}
 else {rmsd <- mean(c(a3,a2))}
  
 return (c(a1,rmsd,l_length,round(100*a2/a1,0)))
}


#------------ compute Mean Absolite Error of the logarythmic plot using bootstrapping
MAE <- function(x0,y0) {
  a<- NULL
  b<- NULL
  d<- NULL
  llist = 1
  sublist=1
  if(length(x0) > 10) { llist =100; sublist = 0.8 }
  for (i in 1:llist) {
    ind <- seq(1,length(x0))
    i <-sample(ind, length(ind)*sublist,replace=T)
    y <- y0[i]
    x <- x0[i]

    fit_correct <- lm(y ~ x)
    cc <-summary(fit_correct)$coefficients[,1]
    mue <- mean(abs(fit_correct$residuals))
    dd2=abs(y-(cc[2]*x+cc[1]))
    k <- (10^(cc[2]*x+cc[1]))/(10^y)
    if(length(k) != length(k[k>1]))    k[k<1] <- 1/k[k<1]

    mueabs <- mean(10^dd2-1)*100
    a <-c(a,mueabs)
    
    dd2 <- abs(x-(y-cc[1])/cc[2])
    k <- (10^((y-cc[1])/cc[2]))/(10^x)
    if(length(k) != length(k[k>1]))  k[k<1] <- 1/k[k<1]

 #   muerel <- round(mean(10^dd2-1)*100,0)
    muerel <- mean(k)
    b <- c(b,muerel)
    r2test <- round(summary(fit_correct)$r.squared,2)
    d<- c(d,r2test)

  }

  return(c(round(mean(a),0),round(mean(b),1),round(mean(d),2)))
}


#=
#------------ compute Mean Absolite Error of the delta(logarythmic) plot using bootstrapping
MAE_diff <- function(x0,y0,ref) {
  a<- NULL
  b<- NULL
  d<- NULL
  llist = 1
  sublist=1
  nl <- which(is.na(y0))
  if(length(x0) > 10) { llist =100; sublist = 0.8 }
  for (i in 1:llist) {
    ind <- seq(1,length(x0))
    i <-sample(ind, length(ind)*sublist,replace=T)
  
    y <- y0[i]
    x <- x0[i]

    fit_correct <- lm(y ~ x)
    cc <-summary(fit_correct)$coefficients[,1]
    mue <- mean(abs(fit_correct$residuals))
    yfit <- (cc[2]*x+cc[1]) 
    dd2=abs(y-yfit)+yfit
    mueabs <- mean(ref*(10^dd2-1)/(10^yfit))*100
    a <-c(a,mueabs)
    
    dd2 <- abs(x-(y-cc[1])/cc[2])
    k <- (10^((y-cc[1])/cc[2]))/(10^x)
    if(length(k) != length(k[k>1]))  k[k<1] <- 1/k[k<1]
    
    muerel <- mean(k)
    b <- c(b,muerel)
    r2test <- round(summary(fit_correct)$r.squared,2)
    d<- c(d,r2test)
    
  }
  return(c(round(mean(a),0),round(mean(b),1),round(mean(d),2)))
}




#----------------- PLOT FUNCTION RAMD VS EXP. KOFF FROM THE EXEL TABLE
make_complete_set_ramd <- function(exp_org,exclude_name, exclude_data) {
  exp <-  na.omit(exp_org)


  xe <-function(ee) return(log10(1/ee$koff))
  ye <-function(ee) return(log10(ee$ramd))
  xe0 <- function(ee,n) return(log10((1/(ee$koff+n*ee$koff_SD))))
  ye0 <- function(ee,n) return(log10(ee$ramd+n*ee$ramd_SD))
  Stat_print <- function(fit) {
  print(paste("R2= ",round(summary(fit)$r.squared,2),"residual= ",round(mean(abs(resid(fit))),2),"; fitting on ",summary(fit)$df[2]," degrees of freedom  "))
   }

  e  <-  na.omit(exp)
  print("")
  print("=====================================================================")
  print("=======Statistical analysis of a complete set of compounds ===========")
  print("Complete set:")
  print(e$name)
  fit <- lm(ye(e) ~ xe(e))
  Stat_print(fit)
  
  #========= this is for HSP90 to exclude quinazoline compounds from statistical analysis
  # delete for other targets
  e_cl <- e
  if(length(exclude_data) > 0 && (exclude_name !="")) {
    print("=====================================================================")
    print("=======Statistical analysis of a subset without outliers defined in the input file  ===========")
    print("===== A subset to be excluded: ")
    print(paste(exclude_name,"=",exclude_data,sep=" "))
    for (k in seq_along(exclude_data)) { 
        e_cl <- subset(e_cl, e_cl[,exclude_name] != exclude_data[k])
    }
    print("Current set:")
    print(e_cl$name)
    fit <- lm(ye(e_cl) ~ xe(e_cl))
    Stat_print (fit)
  #=====================================================================
  
#  cc <-summary(fit)$coefficients[,1]
#  mue <- round(mean(abs(fit$residuals)),2)
    mm <- MAE(xe(e_cl),ye(e_cl))
    mueabs=mm[1]
    muerel=mm[2]
    r2=mm[3]
    print ("Bootstrapping:")
    print(paste("R2=",r2,"Mean Ansigned  Error (MAE)=  ",mueabs,"%","Mean Prediction Uncertainty(MPU)= ",muerel,"folds",sep=" "))
  }
  #============= exclude outliers and make statistical analysis once more=====================
  e_cl$cooks<- cooks.distance(fit)
  meancook <- mean(e_cl$cooks)
  print(paste("mean  cooks distances=",round(meancook,3),sep=" "))
#  e$resi <- resid(fit)
#  e$rstand<- rstandard(fit)
#  e$rstudent<- rstudent(fit)
#  e$dfbeta<- dfbeta(fit)
#  print(subset(e, abs(e$rstand) > 1.6))
  print("")
  print("=====================================================================")
  print("========Statistical analysis of a subset without outliers defined by the Cook distance (and thoese defined in the input file)===========")
  print("Outliers-  cook's distances > 4")
  print("list of outliers:")
#  e_durty <- subset(e, abs(e$rstand) > 1.6 )
  e_clean <- subset(e_cl, e_cl$cooks < 3*meancook  ) 
  e_durty <- subset(e_cl, e_cl$cooks > 3*meancook )
  print(e_durty$name)
  print("Current set:")
  print(e_clean$name)
  fit_correct <- lm(ye(e_clean) ~ xe(e_clean))
  Stat_print (fit_correct)
  cc <-summary(fit_correct)$coefficients[,1]
 # mue <- round(mean(abs(fit_correct$residuals)),2)
  mm <- MAE(xe(e_clean),ye(e_clean))
  mueabs=mm[1]
  muerel=mm[2]
  r2=mm[3]
  print ("Bootstrapping:")
  print(paste("R2 =",r2," MAE=",mueabs,"%"," MPU = ",muerel," folds",sep=" "))
  e$MPU<- (10^((ye(e)-cc[1])/cc[2]))/(10^xe(e))
  write.xlsx(e, "output-data.xlsx")  
  
}  

paper <- function(exp_name,dirf="./",sheetName = "") {
  print(exp_name)
  print(dirf)
  print(sheetName)
  exp <- read.xlsx(exp_name, sheetName = sheetName)
  cbind(exp,replicas=0)
  e <- exp
  cbind(e,replicas=0)
  cbind(e,SD_replicas=0)
  
  for (k in 1:length(e$filename)) {
    file_list <- list.files(path=dirf,pattern=paste(e$filename[k],"_",sep=""),full.names =TRUE)
    print("============================")
    print (c(toString(e$name[k]),toString(e$filename[k])))
    if (length(file_list) > 0) {
      out <-one_compound(file_list,e$name[k])
      e$ramd[k] <- out[1]
      e$ramd_SD[k] <- round(out[2],2)
      e$replicas[k]<- out[3]
      e$SD_replicas[k]<- round(out[4],3)
    }
    else { e$ramd[k] <- NA}
  }
  write.xlsx(e, "output-data.xlsx")
  return (e)
}


loading <- function() {
  library(xlsx) 
  library(Hmisc)
}

#Input: 
#1. Exel file with compound and file names and experimental data (all data are in the tab called "EXP")
#2. a set of data from the directory "DATA"     
args = commandArgs(trailingOnly = TRUE)
if (length(args)< 1) {
  print("Usage:  TRJ-Analysis-R input_file ")
#example of an input file 
#table RAMD.xlsx
#tab EXP
#data DATA
#exclude_name scaffold
#exclude_data H,F,G,M 
#
} else {
    if (file.exists(args[1])) {
        loading()
        t_table ="RAMD.xlsx"
        t_tab="Sheet1"
        t_data="./"
        t_exclude_name=""
        t_exclude_data=""
        dat<-read.table(args[1],header=FALSE)
        print("======= Input DATA ===========")
        print(dat)
        for(i in seq_along(dat$'V1')) {
	             if(dat$'V1'[i]=='table') {
                                    t_table=toString(dat$'V2'[i])
                                    print(paste("EXEL file:", t_table,sep="  "))
                      }
                     if(dat$'V1'[i]=='tab') {
                                    t_tab=toString(dat$'V2'[i])
                                    print(paste(" TAB name in the EXEL file :", t_tab,sep="  "))
                      }
                     if(dat$'V1'[i]=='data') {
                                     t_data=toString(dat$'V2'[i])
                                    print(paste(" dir name :", t_data,sep="  "))
                      }
                     if(dat$'V1'[i]=='exclude_name') {
                                     t_exclude_name=toString(dat$'V2'[i])
                                    print(paste(" name of the data to be excluded :", t_exclude_name,sep="  "))
                      }
                     if(dat$'V1'[i]=='exclude_data') {
                                     t_exclude_data=unlist(strsplit(toString(dat$'V2'[i]),","))
                                    print(paste(" data to be excluded :", t_exclude_data,sep="  "))
                      }
                     }
##       print(table)
       exp <-paper(t_table,t_data,t_tab) # compute residence times for 
          if("koff" %in% colnames(exp)) {
               make_complete_set_ramd(exp,t_exclude_name,t_exclude_data) #make a statistics of computed vs experimental reesidence time
         }
       }
    else print("error reading the input file. Correct Usage:  TRJ-Analysis-R input_file ")
}



